<?php

namespace ObsMania;

use ManiaControl\Settings\SettingManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListner;
use ManiaControl\Commands\CommandListener;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Maps\MapManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Callbacks\Models\RecordCallback;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Utils\Formatter;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Admin\AuthenticationManager;
use FML\Controls\Control;
use FML\Controls\Frame;
use FML\Controls\Label;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Quad;
use FML\Controls\Entry;
use FML\Controls\Quads\Quad_BgsPlayerCard;
use FML\Controls\Quads\Quad_Icons128x128_1;
use FML\Controls\Quads\Quad_BgRaceScore2;
use FML\ManiaLink;
use FML\Script\Features\Paging;
use ManiaControl\Bills\BillManager;
use ManiaControl\Files\FileUtil;

class SprintLiveRanking implements CallbackListener, CommandListener, ManialinkPageAnswerListener, TimerListener, Plugin {


  const ID = 96;
  const VERSION = 0.1;
  const NAME = 'ObsMania Sprint LiveRanking';
  const AUTHOR = 'MarlBurroW';
  const DESCRIPTION = 'Display a live ranking widget for the sprint mode (by Guerro)';

  const WIDGET_MLID = "obsmania_sprint_liveranking_widget";
  const CB_PLAYERSSCORES      = "Sprint_PlayersScores";

  private $maniaControl = null;
  private $ranking = array();
  private $players = array();

  private $widgetStyle = "BgRaceScore2";
  private $widgetSubstyle = "BgCardPlayer";

  private $headerStyle = "BgRaceScore2";
  private $headerSubstyle = "CupFinisher";

  private $rowStyle = "BgRaceScore2";
  private $rowSubstyle = "BgCardPlayer";



  /**
   * Return the ID of the plugin (Plugin interface)
   * @return [type] [description]
   */
  public static function getId() {
    return self::ID;
  }

  /**
   * Return the name of the plugin (Plugin interface)
   * @return [type] [description]
   */
  public static function getName() {
    return self::NAME;
  }

  /**
   * Return the version of the plugin (Plugin interface)
   * @return [type] [description]
   */
  public static function getVersion() {
    return self::VERSION;
  }

  /**
   * Return the author of the plugin (Plugin interface)
   * @return [type] [description]
   */
  public static function getAuthor() {
    return self::AUTHOR;
  }

  /**
   * Return the description of the plugin (Plugin interface)
   * @return [type] [description]
   */
  public static function getDescription() {
    return self::DESCRIPTION;
  }

  /**
   * Prepare method (Plugin interface)
   * @param  ManiaControl $mc [description]
   * @return [type]           [description]
   */
  public static function prepare(ManiaControl $mc) {

  }

  /**
   * Load the plugin (Plugin interface)
   * @param  ManiaControl $mc [description]
   * @return [type]           [description]
   */
  public function load(ManiaControl $mc) {

    // Get the Maniacontrol instance
    $this->maniaControl = $mc;

    // Settings
    $this->maniaControl->settingManager->initSetting($this, "Title", "Live Ranking");
    $this->maniaControl->settingManager->initSetting($this, "Position X", -138);
    $this->maniaControl->settingManager->initSetting($this, "Position Y", 80);
    $this->maniaControl->settingManager->initSetting($this, "Rows displayed", 12);

    // Common Callbacks
    $this->maniaControl->callbackManager->registerCallbackListener(SettingManager::CB_SETTING_CHANGED, $this, 'handleSettingChanged');
    $this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnect');
    $this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'handlePlayerDisconnect');
    $this->maniaControl->callbackManager->registerCallbackListener(Callbacks::BEGINMAP, $this, 'handleBeginMap');
    $this->maniaControl->callbackManager->registerCallbackListener(Callbacks::ENDMAP, $this, 'handleEndMap');
    $this->maniaControl->callbackManager->registerCallbackListener(Callbacks::SCRIPTCALLBACK, $this, 'handleScriptCallback');

    // Sprint Script Callbacks
    $this->maniaControl->callbackManager->registerCallbackListener(self::CB_PLAYERSSCORES, $this, 'handlePlayersScores');


    $this->displayLiveRanking();

  }

  /**
   * Callback triggered when a scriptcallback is received (any), and trigger the good local callback
   * @param  [type] $name [description]
   * @param  [type] $data [description]
   * @return [type]       [description]
   */
  public function handleScriptCallback($name, $data) {
		switch ($name) {
			case self::CB_PLAYERSSCORES:
				$this->maniaControl->getCallbackManager()->triggerCallback(self::CB_PLAYERSSCORES, $data);
				break;
    }
  }

  /**
   * Unload the plugin (Plugin interface)
   * @return [type] [description]
   */
  public function unload() {
    $this->hideLiveRanking();
    $this->maniaControl = null;

  }

  /**
   * PlayerScore callback
   * @param  [type] $data [description]
   * @return [type]       [description]
   */
  public function handlePlayersScores($data){
    // $data = '[
    //   {"nickname":"Player1","playerlogin":"player1","score":3, "temp":1},
    //   {"nickname":"Player2","playerlogin":"player2","score":10, "temp":2},
    //   {"nickname":"Player3","playerlogin":"player3","score":5, "temp":5},
    //   {"nickname":"Player4","playerlogin":"player4","score":8, "temp":4},
    //   {"nickname":"Player5","playerlogin":"player5","score":1, "temp":3}
    //   ]';
    $this->ranking = json_decode($data);

    $this->displayLiveRanking();
  }

  /**
   * Setting changes callback
   * @param  ManiaControlSettingsSetting $setting [description]
   * @return [type]                               [description]
   */
  public function handleSettingChanged(\ManiaControl\Settings\Setting $setting) {
    $this->displayLiveRanking();
  }

  /**
   * Player connect callback
   * @param  Player $player [description]
   * @return [type]         [description]
   */
  public function handlePlayerConnect(Player $player){
    $this->displayLiveRanking();
  }

  /**
   * Player disconnect callback
   * @param  Player $player [description]
   * @return [type]         [description]
   */
  public function handlePlayerDisconnect(Player $player){
    $this->displayLiveRanking();
  }

  /**
   * Map begin callback
   * @return [type] [description]
   */
  public function handleBeginMap(){
    $this->ranking = array();


    $this->displayLiveRanking();
  }

  /**
   * Map end callback
   * @return void
   */
  public function handleEndMap(){

    $this->displayLiveRanking();
  }

  /**
   * Create a rank row for the live ranking
   * @param  string $rank     [description]
   * @param  string $nickname [description]
   * @param  string $score    [description]
   * @param  float $ypos      Y Position of the row, 0 is top
   * @return Frame            Return a Frame object
   */
  public function createRow($rank, $nickname, $score, $ypos){


    $rowFrame = new Frame();

    $rowFrame->setPosition(-20, $ypos, 10 );
    $rowFrame->setSize(50,5);

    $rankLabel = new Label();
    $rowFrame->add($rankLabel);
    $rankLabel->setHAlign(Control::CENTER);
    $rankLabel->setX(0);
    $rankLabel->setSize(5, 5);
    $rankLabel->setTextSize(1.5);
    if(is_int($rank)){
      switch($rank){
        case 1:
          $rankLabel->setText('$FF0' . $rank);
        break;
        case 1:
          $rankLabel->setText('$CCC' . $rank);
        break;
        case 1:
          $rankLabel->setText('$963' . $rank);
        break;
        default:
          $rankLabel->setText('$39F' . $rank);
        break;
      }
    }else{
      $rankLabel->setText($rank);
    }
    $rankLabel->setStyle('TextButtonBig');

    $nicknameLabel = new Label();
    $rowFrame->add($nicknameLabel);
    $nicknameLabel->setHAlign(Control::LEFT);
    $nicknameLabel->setX(5);
    $nicknameLabel->setSize(25, 5);
    $nicknameLabel->setTextSize(1);
    $nicknameLabel->setText($nickname);
    $nicknameLabel->setTextEmboss(true);
    $nicknameLabel->setStyle('TextButtonBig');

    $scoreLabel = new Label();
    $rowFrame->add($scoreLabel);
    $scoreLabel->setHAlign(Control::CENTER);
    $scoreLabel->setX(38);
    $scoreLabel->setSize(10, 5);
    $scoreLabel->setTextSize(1.5);
    $scoreLabel->setText($score);
    $scoreLabel->setStyle('TextButtonBig');



    return $rowFrame;

  }

  /**
   * Hide the widget Manialink
   * @param  [type] $playerlogin [description]
   * @return void           [description]
   */
  public function hideLiveRanking($playerlogin = null){
    $this->maniaControl->getManialinkManager()->hideManialink(self::WIDGET_MLID, $playerlogin);
  }

  /**
   * Display the liveranking Widget
   * @param  [type] $playerlogin [description]
   * @return void             [description]
   */
  public function displayLiveRanking($playerlogin = null){

    $this->updateRankingArray();

    $title = $this->maniaControl->settingManager->getSettingValue($this, "Title");
    $pos_x = $this->maniaControl->settingManager->getSettingValue($this, "Position X");
    $pos_y = $this->maniaControl->settingManager->getSettingValue($this, "Position Y");
    $rows_displayed = $this->maniaControl->settingManager->getSettingValue($this, "Rows displayed");

    $manialink = new ManiaLink(self::WIDGET_MLID);

    $size_y = 15 + ($rows_displayed * 5);

    $frame     = new Frame();
    $manialink->add($frame);

    $frame->setPosition($pos_x, $pos_y);

    $background = new Quad();

    $background->setVAlign(Control::TOP);
    $background->setStyles($this->widgetStyle, $this->widgetSubstyle);
    $frame->add($background);
    $background->setSize(50,$size_y);

    $ypos = -5;
    $rankCount = 1;

    $rowTitleFrame = new Frame();
    $rowTitleFrame->setPosition(0, $ypos);
    $scoreLabel = new Label();
    $rowTitleFrame->add($scoreLabel);
    $scoreLabel->setHAlign(Control::CENTER);
    $scoreLabel->setX(0);
    $scoreLabel->setSize(50, 5);
    $scoreLabel->setTextSize(2);
    $scoreLabel->setText($title);
    $scoreLabel->setStyle('TextButtonBig');
    $frame->add($rowTitleFrame);

    $ypos -= 8;

    $rowFrame = $this->createRow("#","Nickname","Score", $ypos);

    $rowBackground = new Quad();
    $rowBackground->setSize(50,5);
    $rowBackground->setX(20);
    $rowBackground->setStyles($this->headerStyle, $this->headerSubstyle);
    $rowFrame->add($rowBackground);

    $frame->add($rowFrame);
    $ypos-= 5;

    $rowcounter = 1;

    foreach ($this->ranking as $key => $rank) {
      if($rowcounter > $rows_displayed)
        break;

      $nickname = $rank->nickname;

      if ($rank->score !== false){
        $score = '$o$w' + ($rank->score + $rank->temp)  . '$z ($0f0+' . $rank->temp . '$z)';
        $rnk = $rankCount;
        $rankCount++;
      }
      else{
        $score = '$o$w-';
        $rnk = '$o$w-';
      }


      $rowFrame = $this->createRow($rnk,$nickname,$score, $ypos);

      $rowBackground = new Quad();
      $rowBackground->setSize(50,5);
      $rowBackground->setX(20);
      $rowBackground->setStyles($this->rowStyle, $this->rowSubstyle);
      $rowFrame->add($rowBackground);

      $frame->add($rowFrame);

      $ypos-= 5;
      $rowcounter++;
    }



    $this->maniaControl->getManialinkManager()->sendManialink($manialink, $playerlogin);

  }




  public function refreshPlayerList(){


      $players = $this->maniaControl->playerManager->getPlayers();

      if (empty($players)) {
          return array();
      }

      $playersInfo = array();

      foreach ($players as $player) {
          $info = new \stdClass();
          $info->playerlogin = $player->login;
          $info->nickname = $player->nickname;

          array_push($playersInfo, $info);
      }

      $this->players = $playersInfo;
      return $playersInfo;
  }

  public function updateRankingArray(){
    $this->refreshPlayerList();

    // For each player
    foreach($this->players as $pkey => $player){


      $playerExistInRanking = false;
      // Check if exist in ranking
      foreach ($this->ranking as $rkey => $rank) {
        if($player->playerlogin == $rank->playerlogin)
          $playerExistInRanking = true;
      }
      // If player doesn't exists in ranking, adding it with no score and temp
      if(!$playerExistInRanking){
        $newRank = $player;
        $newRank->score = false;
        $newRank->temp = false;
        array_push($this->ranking, $newRank);
      }

    }

    usort($this->ranking, array($this, "orderByRankDesc"));

  }


  /**
   * Sorting method that order the ranking by Rank Desc
   * @param  [type] $a [description]
   * @param  [type] $b [description]
   * @return [type]    [description]
   */
  private function orderByRankDesc($b, $a)
  {
      $a_vscore = $a->score + $a->temp;
      $b_vscore = $b->score + $b->temp;

      return $a_vscore  - $b_vscore;
  }


}
